/* SQL Database Interface header. 
   Author: adv (Andriy Marchenko), 06 sep, 2001
           < adv@adv.org.ua >  */

#ifdef __cplusplus 
extern "C"
{
#endif

#include "config.h"
#include "types.h"


typedef struct __sql_opt_t
{
  const char* keyword;
  const char* value;
  int         ivalue;

  struct __sql_opt_t* next;
} sql_opt_t;


typedef struct __sql_t
{
  void* env;     // ora
  void* err;     // ora
  void* server;  // ora
  void* svc;     // ora

  void* connection;

  sql_opt_t* opt;
  struct __sql_res_t* res_last;

  __int16_t   (*connect)     (struct __sql_t* drv);
  void        (*disconnect)  (struct __sql_t* drv);
  void        (*freeres)     (struct __sql_res_t* res);
  struct __sql_res_t* (*execute)(struct __sql_res_t* res, const char* query);
  __int16_t   (*fetch)       (struct __sql_res_t* res);
  const char* (*getfield_pos)(struct __sql_res_t* res, __uint32_t field);
  const char* (*getfield)    (struct __sql_res_t* res, const char* field);
  __uint32_t  (*fieldlen)    (struct __sql_res_t* res, __uint32_t field);

  __uint32_t  (*numrows)     (struct __sql_res_t* res);
  __uint32_t  (*numcols)     (struct __sql_res_t* res);
  const char* (*fieldname)   (struct __sql_res_t* res, __uint32_t field);
  __uint32_t  (*lastrec)     (struct __sql_t* drv);

} sql_t;


typedef struct __sql_res_t
{
  sql_t* drv;

  void* result;
  void** row;
  void* field;

  __int32_t  current_row;
  __uint32_t num_cols;
  __uint32_t  num_rows;

  struct __sql_res_t* next;
  struct __sql_res_t* prev;
const char* q;
} sql_res_t;


API_SDBI sql_t*      sql_init        (const char* driver);
API_SDBI void        sql_close       (sql_t* drv);

API_SDBI __uint16_t  sql_set_opt     (sql_t* drv, const char* keyword, const char* value);
API_SDBI const char* sql_get_opt     (sql_t* drv, const char* keyword);
API_SDBI __int32_t   sql_get_iopt    (sql_t* drv, const char* keyword);

API_SDBI __uint16_t  sql_connect     (sql_t* drv);
API_SDBI void        sql_disconnect  (sql_t* drv);

API_SDBI sql_res_t*  sql_execute     (sql_t* drv, const char* query);
API_SDBI sql_res_t*  sql_freeres     (sql_res_t* res);
API_SDBI __int16_t   sql_fetch       (sql_res_t* res);

API_SDBI const char* sql_getfield_pos(sql_res_t* res, __uint32_t field);
API_SDBI const char* sql_getfield    (sql_res_t* res, const char* field);
API_SDBI __uint32_t  sql_fieldlen    (sql_res_t* res, __uint32_t field);
API_SDBI __uint32_t  sql_lastrec     (sql_t* drv);

API_SDBI __uint32_t  sql_numrows     (sql_res_t* res);
API_SDBI __uint32_t  sql_numcols     (sql_res_t* res);
API_SDBI const char* sql_fieldname   (sql_res_t* res, __uint32_t field);

#ifdef __cplusplus 
}
#endif
